package HSPC::Localization;

use strict;

use HSPC::Config;
use HSPC::LocResource;
use POSIX qw(locale_h);
use XML::Simple;
use HSPC::Logger qw(sw_die sw_log);
use HSPC::Util qw(sw_purify_data);
use HSPC::Math qw(round2);
use Fcntl ':flock';
use locale;

our (%LOCALE, $IS_LOADED);

use base qw(Exporter);
our @EXPORT = qw(
	string
	hash
	array
	boptions
	string_lang
	get_languages
	string_for_lang
	is_string_exist
	loc_string
	safe_setlocale
	reset_locale
	%LOCALE
);
our @EXPORT_OK = qw(
	string
	data_size_kb_to_str lang_field_select_expr
	set_language get_language current_lang
);

use constant LOCALE_LOCK => '/var/log/hspc/locale.lock';

## < current lang
sub current_lang {
	return $LOCALE{local_lang} || $LOCALE{def_lang} || 'en';
}

## > value
## < value in representation form
sub data_size_kb_to_str {
	my $value = shift;
	$value = 0 unless ( $value );
	## XXX AMARK canot use DataCheck due to cyclic use
	unless (
		$value =~ /^[+-]?\d+\.?\d*$/ 	## is a real number
		|| $value =~ /^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/ 	## is a C float
	) {
		return $value;
	}
	my $count = 0;## num of 2^10 multipliers
	while ( $value >= 1024 ) {
		$value = $value/1024;
		$count++;
		last if ( $count == 2 );
	}
	$value = round2($value);
	my $string_id = 'num_kb';
	if ( $count == 1 ) {
		$string_id = 'num_mb';
	} elsif ( $count == 2 ) {
		$string_id = 'num_gb';
	}
	return string($string_id,n=>$value);
}##/data_size_kb_to_str

## >> name
## >> lang
## < SQL select_expr
sub lang_field_select_expr {
	my ($name,$lang) = @_;
	sw_die("lang_field_select_expr: name and lang should be present.") unless (
		$name && $lang
	);

	my $str = "IF(LENGTH(SUBSTR($name,LOCATE('<lang_$lang>',$name)+10,".
		"LOCATE('</lang_$lang>',$name)-LOCATE('<lang_$lang>',$name)-11)),".
		"SUBSTR($name,LOCATE('<lang_$lang>',$name)+10,LOCATE('</lang_$lang>',".
		"$name)-LOCATE('<lang_$lang>',$name)-11),SUBSTR($name,".
		"LOCATE('<lang_def>',$name)+11,LOCATE('</lang_def>',$name)".
		"-LOCATE('<lang_def>',$name)-12))";

	return $str;
}##/lang_field_select_expr

## ########################
my %strings;
my %info;
my %geo_countries;
my %geo_states_us;
my %geo_states_ca;
my %geo_states_mx;

## ########################
my (%include_langs, %local_languages);
my (%include_geo_countries, %include_geo_states_us, %include_geo_states_ca, %include_geo_states_mx);
my (@config_langs, %config_langs);

my $default_path = '/var/opt/hspc-root/i18n';	#HARDCODE

# HARDCODE
my $def_lang_id = 'en';

sub load_language {
	my $lang = lc(shift);
	my $lang_data = shift;

	$strings{$lang} = {};
	$include_langs{$lang} = $strings{$lang};

	$info{$lang} = {};
	$local_languages{$lang} = $lang_data;

	$geo_countries{$lang} = [];
	$include_geo_countries{$lang} = $geo_countries{$lang};

	$geo_states_us{$lang} = [];
	$include_geo_states_us{$lang} = $geo_states_us{$lang};

	$geo_states_ca{$lang} = [];
	$include_geo_states_ca{$lang} = $geo_states_ca{$lang};

	$geo_states_mx{$lang} = [];
	$include_geo_states_mx{$lang} = $geo_states_mx{$lang};

	my $result = opendir LPMDIR, $default_path.'/'.uc($lang);
	my @default_files = map { /^([\w\-]+)\.xml$/; $1 } grep { /^[\w\-]+.xml$/ } readdir( LPMDIR );
	closedir LPMDIR;
	my $cust_path = $default_path;
	$cust_path =~ s/i18n/custom\/i18n/;
	$result = opendir LPMDIR, $cust_path.'/'.uc($lang);
	my @custom_files;
	if ( $result ) {
		push @custom_files, map { /^([\w\-]+)\.xml$/; $1 } grep { /^[\w\-]+.xml$/ } readdir( LPMDIR );
		closedir LPMDIR;
	}

	my $lres;
	my $charset = $local_languages{$lang}->{utf8_map};
	my ($skey,$svalue);
	my $lang_geo_data = {
		'countries' => $geo_countries{$lang},
		'states_us' => $geo_states_us{$lang},
		'states_ca' => $geo_states_ca{$lang},
		'states_mx' => $geo_states_mx{$lang},	
	};

	my $i = 0;
	foreach my $cf (@default_files,@custom_files) {
		my $path = $default_path;
		$path = $cust_path if ( $i > $#default_files );
		$i++;
		$lres = HSPC::LocResource->new($path.'/'.uc($lang).'/'.$cf.'.xml', $lang, $charset);
		my $strings_hash = $lres->strings();
		if ( $cf eq 'strings' ) {
			while(($skey,$svalue) = each(%{$strings_hash})) {
				$include_langs{$lang}->{$skey} = $svalue;
			}
		}
		elsif ( $cf eq 'countries' 
			|| $cf eq 'states_us'
			|| $cf eq 'states_ca'
			|| $cf eq 'states_mx') 
		{
			while(($skey,$svalue) = each(%{$strings_hash})) {
				$skey =~ /(\w{2})$/;
				$skey = $1;
				push(@{$lang_geo_data->{$cf}},[uc($skey),$svalue]);
			}
		}
		else {
			while(($skey,$svalue) = each(%{$strings_hash})) {
				exists($include_langs{$lang}->{$skey}) 
					|| ( $include_langs{$lang}->{$skey} = $svalue );
			}
		}
		if ( $lres->convert() ) {
			$local_languages{$lang}->{'charset'} = 'UTF-8';
		}
	}
	
	
	my $current_locale = safe_setlocale(LC_COLLATE, $local_languages{$lang}->{posixlocale});
	@{$geo_countries{$lang}} = sort { $a->[1] cmp $b->[1] } @{$geo_countries{$lang}};
	@{$geo_states_us{$lang}} = sort { $a->[1] cmp $b->[1] } @{$geo_states_us{$lang}};
	@{$geo_states_ca{$lang}} = sort { $a->[1] cmp $b->[1] } @{$geo_states_ca{$lang}};
	@{$geo_states_mx{$lang}} = sort { $a->[1] cmp $b->[1] } @{$geo_states_mx{$lang}};
	reset_locale(LC_COLLATE,$current_locale);
}

# > lang_id
sub set_language {
	my $lang_id = shift;
	$LOCALE{local_lang} = $lang_id || $LOCALE{def_lang};

	my $lang_conf = get_language($LOCALE{local_lang});

	$LOCALE{local_lang_conf} = $lang_conf;
	$LOCALE{local_charset} = $lang_conf
		? $lang_conf->{charset}
		: 'UTF-8';
}


sub load_languages_list {
	my $langs = {};
	foreach my $ld ( glob("$default_path/*") ) {
		## load file
		$ld =~ /^$default_path\/(.*)$/;
		my $id = lc($1);
		next unless ( $id =~ /^[a-z]{2,2}$/ ); ## not language id
		my $lang_fcontent = load_file("$ld/language.xml");
		$lang_fcontent = qq|<?xml version="1.0" encoding="iso-8859-1"?>
			<language id="$id" title="Unknown">
				<title_id>lang_$id\_uc</title_id>
				<flag_icon_id>flag_$id</flag_icon_id>
				<dateformat>%d-%b-%Y</dateformat>
				<datetimeformat>%d-%b-%Y, %H:%M</datetimeformat>
				<timeformat>%H:%M</timeformat>
		    </language>
		| unless $lang_fcontent;

		my $lang = XMLin($lang_fcontent);
		die "Unable to parse the language $id file" unless $lang;
	
		## remove 'not ready yet' language entries
		$lang = sw_purify_data($lang);
		$langs->{$id} = $lang;
	}

	## add "id" attribute
	map $langs->{$_}->{id} = $_, keys %$langs;

	return $langs;
}

sub load_all_languages {
	my $langs = &load_languages_list();

	@config_langs = keys %$langs;
	%config_langs = map { $_ => 1 } @config_langs;


	foreach (@config_langs) {
		load_language($_,$langs->{$_});
	}
	return 1;
}

sub main {
	$IS_LOADED = 1;

	## Check for module running under Apache
	return load_all_languages()
		if (grep { 'HSPC/Startup.pm' eq $_ } keys %INC);

	## Not all - means we're under console and need only default language
	my $langs = &load_languages_list();
	$langs = {
		map { $_, $langs->{$_} }
			grep lc($LOCALE{def_lang}) eq lc($_),
				keys %$langs
	};

	@config_langs = keys %$langs;
	%config_langs = map { $_ => 1 } @config_langs;

	foreach (@config_langs) {
		load_language($_,$langs->{$_});
	}
	return 1;
}

### /include language files ####


&main()
	unless $IS_LOADED;

##########################

# > string_id : REQ
sub string{
	my $id = shift;
	my %param = (@_);

	return string_for_lang($LOCALE{local_lang}, $id, %param);
}

# > string_id : REQ
sub is_string_exist{
	my $id = shift;
	my $lang_id = $LOCALE{local_lang} || $def_lang_id;
	return 1 if($include_langs{$lang_id}->{$id});
	return 0;
}

## > lang
## > string_id : REQ
sub string_for_lang{	
	my $lang = shift;
	my $id = shift;
	my %param = (@_);

	## extract parameters from ID string
	if($id =~ / /) {
		my $param_str;
		($id,$param_str) = split(/\s+/,$id,2);
		while($param_str =~ s/(\w+)="(|.*?[^\\])"//s) {
			$param{$1}=$2;
		}
	}

	## get string from repository
	my $str = string_lang($lang, $id) ||
		return "Error: STRING ($id) NOT FOUND!";

	## replace parameters
	foreach (keys %param) {
		$str =~ s/\%$_\%/$param{$_}/g;
	}
	
	return $str;
}

# Translates HASH values
# > hasref : REQ
sub hash {
	my $h = shift;
	my $lang = $LOCALE{local_lang};
	map { $h->{$_} = string_lang($lang, $h->{$_}) || 
		"Error: STRING ($h->{$_}) NOT FOUND!" } keys %$h;
}

# Translates ARRAY elements
# > arrayref : REQ
sub array {
	my $a = shift;
	my $lang = $LOCALE{local_lang};
	for (0 .. $#$a) {
		$a->[$_] = string_lang($lang, $a->[$_]) || 
			"Error: STRING ($a->[$_]) NOT FOUND!";
	}
}

# Translates options ARRAY-of-ARRAYREFs
# > arrayref : REQ
sub boptions {
	my $a = shift;
	my $lang = $LOCALE{local_lang};
	map { $_->[1] = string_lang($lang, $_->[1]) || "Error: STRING ($_->[1]) NOT FOUND!" } @$a;
}


# > lang_id : REQ
# > string_id : REQ
##> strict
sub string_lang{
	my $lang_id = shift;    
	my $id=shift;
	my $strict = shift;
	if(!$lang_id){
		$lang_id = $def_lang_id;
	}	
	my $str = $include_langs{$lang_id}->{$id};
	return $str if ( length($str) || $strict );
	return $include_langs{$def_lang_id}->{$id};
};

sub get_languages{
	my @langs = map { $local_languages{$_} } @config_langs;
	return \@langs;
}

# > lang_id
# < lang hash
sub get_language{
	my $lang_id = shift;
	my %arg = (
		default_on_err => undef,
		@_
	);
	foreach my $l (@config_langs){
		my $lang = $local_languages{$l};
		return $lang if $lang->{id} eq $lang_id;
	}
	if ($arg{default_on_err}) {
		return (values %local_languages)[0];
	}

	$ENV{q}->warnings("Language plugin 'hspc-i18n-$lang_id' is missing. Disable or reinstall plugin.")
		if ( ref $ENV{q} eq 'HSPC::Request::Query' && $ENV{q}->can('warnings') );

	return undef;
}

# > array
# < array of strings with deleted starting and ending spaces
sub trim{
	my @ret;
	foreach my $string (@_){
		$string =~ s/^\s+//;
		$string =~ s/\s+$//;
		push @ret, $string;
	}
	return @ret;
}

# > lang_id
# < countries array ref
sub get_countries_lang{
	my $lang_id = shift;
	$lang_id = $def_lang_id unless( exists $include_geo_countries{$lang_id} );
	return $include_geo_countries{$lang_id} if ( exists $include_geo_countries{$lang_id} );
	return [];
}

# > lang_id
# < states_us array ref
sub get_states_us_lang{
	my $lang_id = shift;
	$lang_id = $def_lang_id unless( exists $include_geo_states_us{$lang_id} );
	return $include_geo_states_us{$lang_id} if ( exists $include_geo_states_us{$lang_id} );
	return [];
}

# > lang_id
# < states_ca array ref
sub get_states_ca_lang{
	my $lang_id = shift;
	$lang_id = $def_lang_id unless( exists $include_geo_states_ca{$lang_id} );
	return $include_geo_states_ca{$lang_id} if ( exists $include_geo_states_ca{$lang_id} );
	return [];
}


# > file_path
# < data
sub load_file
{
	my $file_path = shift;
	unless ( open(SRC, "< $file_path" ) ) {
		return undef;
	}

	local $/ = undef;
	my $c = <SRC>;
	close(SRC);

	return $c;
}

## make localization string
## > string_id
## =>> args key=>value for localization
## < object HSPC::String
sub loc_string {
	my $string_id = shift;
	my %args      = @_;

	require HSPC::String;
	return HSPC::String->new( string_id => $string_id, string_args => \%args );
}

## Safely change locale to avoid dangerous locale changes
## > category
## > locale
## < old_locale
sub safe_setlocale {
	my $category  = shift;
	my $locale    = shift;
	open(LOCKFILE, ">" . LOCALE_LOCK) or die "Can't open lock file: $!";
	flock( LOCKFILE, LOCK_EX );
	seek( LOCKFILE, 0, 2 );
	my $old_locale = setlocale($category);
	sw_log("Locale for category $category is changed from $old_locale to $locale");
	setlocale( $category, $locale );
	return $old_locale;
}

## Reset locale and unlock file
sub reset_locale {
	my $category  = shift;
	my $locale    = shift;
	sw_log("Reset locale for category $category to $locale");
	setlocale( $category, $locale );	
	setlocale( &POSIX::LC_NUMERIC, "C" );	
	flock( LOCKFILE, LOCK_UN );
	close(LOCKFILE);
	unlink(LOCALE_LOCK);
}

1;
