package HSPC::Localization::Date;

use vars qw(@ISA @EXPORT_OK);

use Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(
 &gmt_now &date_now &time_to_date &date_to_str
 &interval_number &period_to_str &duration_to_str &time_to_strtime
 &time_gmt_to_local &time_local_to_gmt &time_to_str &split_datetime
 &str_to_time &time_to_secs &secs_to_time &cmp_times &cmp_dates &min_time
 &max_time &date_diff
);

use HSPC::Localization;
use HSPC::Math qw(round2);
use POSIX();
use Date::Manip;
use HSPC::Config;
use HSPC::Localization;

use constant _interval_names => {
		'second'	=> 1,
		'minute'	=> 60,
		'hour'		=> 3600,
		'day'		=> 86400,
		'week'		=> 604800,
		'month'		=> 2592000,
		'quarter'	=> 7776000,
		'year'		=> 31104000,
		'twoyears'	=> 62208000
};

sub gmt_now { # returns string with current GMT time yyyymmddhhmmss
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime(time);
	return (sprintf("%4.4d-%2.2d-%2.2d %2.2d:%2.2d:%2.2d",
					 $year+1900, $mon+1, $mday, $hour, $min, $sec));
}

sub date_now { # returns string with current GMT date yyyymmdd
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime(time);
	return (sprintf("%4.4d-%2.2d-%2.2d", $year+1900, $mon+1, $mday));
}

## convert full datetime to date only
## >> datetime
## << date
sub time_to_date{
	my $time = shift;
	$time=~s/( .+)//;
	return $time;
}


# >> raw_time 	# REQ_Y #
# >  default	# REQ_N - default value if raw_time is invalid. '' if omitted.
# >  gmtdelta : offset from gmt-time
sub date_to_str{
	my $raw_time = shift;
	my $default = shift || '';
	my $gmtdelta = shift || '';
	my $format = shift || '';
	my $lang_config = $LOCALE{local_lang_conf};
	return $default unless $raw_time;
	return $default if ($raw_time eq '0000-00-00');
	return $default if ($raw_time eq '0000-00-00 00:00:00');

	my $posixlocale = $lang_config->{posixlocale};
	my $charset = $lang_config->{charset};
	$posixlocale =~ s/\..+$/.$charset/;
	my $oldlocale = safe_setlocale( POSIX::LC_ALL, $posixlocale );

	my $str;
	my $secs = time_to_secs( $raw_time);
	my @time_arr = split(' ',$raw_time);
	if (scalar(@time_arr)>1) {
		## $raw_time is datetime - have to be localized
		my $secs = time_to_secs( $raw_time);
		local $ENV{TZ} = $ENV{system_obj} ?
			$ENV{system_obj}->locale_timezone : '/usr/share/zoneinfo/GMT';
		POSIX::tzset();
		my @d = localtime($secs);
		$str = &POSIX::strftime($format||$lang_config->{dateformat}||"%d-%b-%Y", @d);
	} else {
		## $raw_time is date - leave it as is
		local $ENV{TZ} = '/usr/share/zoneinfo/GMT';
		POSIX::tzset();
		my @d = localtime( $secs );
		$str = &POSIX::strftime($format||$lang_config->{dateformat}||"%d-%b-%Y", @d);
	}
	reset_locale( &POSIX::LC_ALL, $oldlocale);
	POSIX::tzset();	
	return $str;
}

## converts interval, defined in seconds to interval defined by interval_name.
## Input:
## 		int		$period in seconds
##		string	Interval name. (possible values see in %interval_names.
## Output:
##		int interval number.
sub interval_number {
	my $period = shift;
	my $interval_name = shift;

	my $interval_sec = &_interval_names->{$interval_name};
	&sw_die("Bad interval name: '$interval_name'") unless $interval_sec;

	return $period/$interval_sec;
}

# > period_value 	# REQ_Y #
sub period_to_str {
	my $period_value = shift;

	my @period = (2592000, 86400, 3600, 60);
	my @ns=(string('second'),string('minute'),string('hour'),string('day'),string('month'));
	my @nm=(string('secs_uc'),string('mins_uc'),string('hours_uc'),string('day_s'),string('months_uc'));

	my @interval=();
	foreach (0..$#period) {
		$interval[4-$_] = int($period_value / $period[$_]);
		$period_value = $period_value % $period[$_];
	}
	$interval[0] = $period_value;

	my @text;
	for(my $i=4; $i>=0; $i--) {
		push @text, $interval[$i].' '.($interval[$i] > 1 ?
			$nm[$i] : $ns[$i]) if $interval[$i];
	}

	return join(' ', @text);
}

sub duration_to_str {
	my $period = shift;

	unless ($period) {
		return '';
	};

	my $duration    = '';

	## !!!!!! don't add hours, days and so on small periods.
	## this function initially created by Billing,
	## and billing need to not show periods in days!

	my $divider     = 2592000; ## month
	my $divider_txt = string('month_s_lc');

	if( $period >= 31104000 ) {
		$divider     = 31104000; ## year
		$divider_txt = string('year_s_lc');
	}

	$duration = round2($period / $divider);

	## drop zeroes after dot.
	$duration = int($duration)
		if( $duration == int($duration) );

	$duration .= ' '.$divider_txt;

	return $duration;
}

####### convert raw gmt datetime to localized string with time in local timezone
## >> raw_time 	# REQ_Y #
## >  timezone
## < return string with localized time, for example :'10.23' or 'AM 10:32'
sub time_to_strtime{
	my $raw_time = shift;
	my $timezone = shift;

	if( (not defined $timezone) ){
		$timezone = $ENV{system_obj}?$ENV{system_obj}->locale_timezone:
			SW_GM_DEF_TIMEZONE;
	}

	my $lang_config = $LOCALE{local_lang_conf};
	my $oldlocale = safe_setlocale( &POSIX::LC_ALL, $lang_config->{posixlocale} );

	my $secs = time_to_secs( $raw_time);
	my $str;

	{
		local $ENV{TZ} = $timezone;
		POSIX::tzset();
		my @d = localtime( $secs );
		$str = &POSIX::strftime($lang_config->{timeformat}||"%H:%M", @d);		
	}
	reset_locale(&POSIX::LC_ALL, $oldlocale);
	POSIX::tzset();	
	return $str;
}

####### convert gmt time to local time according person timezone
## >> raw_time
## >  timezone
## << raw_time
sub time_gmt_to_local{
	my $raw_time = shift;
	my $timezone = shift;

	if( (not defined $timezone) ){
		$timezone = $ENV{system_obj}?$ENV{system_obj}->locale_timezone:
			SW_GM_DEF_TIMEZONE;
	}

	## seconds since 1970 == GMT zone
	my $secs = time_to_secs( $raw_time);
	my $res;
	{
		local $ENV{TZ} = $timezone;
		POSIX::tzset();
		my @d = localtime( $secs );

		$res = POSIX::strftime("%Y-%m-%d %H:%M:%S", @d);
	}
	POSIX::tzset();
	return $res;
}

####### convert local time to gmt time according person timezone
## >> raw_time
## >  timezone
## << raw_time
sub time_local_to_gmt{
	my $raw_time = shift;
	my $timezone = shift;

	if( (not defined $timezone) ){
		$timezone = $ENV{system_obj}?$ENV{system_obj}->locale_timezone:
			SW_GM_DEF_TIMEZONE;
	}

	my $secs = time_to_secs( $raw_time);
	{
		local $ENV{TZ} = $timezone;
		POSIX::tzset();
		my ($sign, $hour, $minute) = unpack("AA2A2", POSIX::strftime("%z", localtime($secs)));
		my $offset = $hour*3600+$minute*60;

		if($sign eq '-'){
			$secs += $offset;
		}else{
			$secs -= $offset;
		}
	}

	POSIX::tzset();
	my @d = gmtime( $secs );

	my $res = POSIX::strftime("%Y-%m-%d %H:%M:%S", @d);

	return $res;
}


####### convert raw datetime to localized string with date & time
## >> raw_time 	# REQ_Y #
## >  gmtdelta : offset from gmt-time
## >  default	# REQ_N - default value if raw_time is invalid. '' if omitted.
## << return string with date & time, for example :'20 Mar 2002 10:23'
##### ATTN : seconds will be dropped
sub time_to_str{
	my $raw_time = shift;
	my $gmtdelta = shift;
	my $default = shift || '';
	return $default unless $raw_time;
	return $default if ($raw_time eq '0000-00-00');
	return $default if ($raw_time eq '0000-00-00 00:00:00');
	my $lang_config = $LOCALE{local_lang_conf};
	my $posixlocale = $lang_config->{posixlocale};
	my $charset = $lang_config->{charset};
	$posixlocale =~ s/\..+$/.$charset/;
	my $oldlocale = safe_setlocale( POSIX::LC_ALL, $posixlocale );

	my $secs = time_to_secs( $raw_time );
	my $str;

	{
		local $ENV{TZ} = $ENV{system_obj}
			? $ENV{system_obj}->locale_timezone
			: ':/usr/share/zoneinfo/GMT';
		POSIX::tzset();
		my @d = localtime( $secs );
		$str = POSIX::strftime($lang_config->{datetimeformat}||"%d.%m.%Y, %H:%M", @d);		
	}
	reset_locale( &POSIX::LC_ALL, $oldlocale);
	POSIX::tzset();	
	return $str;
}
####### extract year from raw datetime
## >> raw_time 	# REQ_Y #
## << return hash with year, for example :'2002', month, ... etc.
sub split_datetime{
	my $raw_time = shift;

	my %date;
	if ($raw_time =~ m/(\d+)-(\S+)-(\d+)(?: (\d+):(\d+))?/){
		$date{year} = $1;
		$date{month} = $2;
		$date{day} = $3;
		$date{hour} = $4;
		$date{minute} = $5;
		return %date;
	}
	return undef;
	## sw_die('invalid datetime argument');
}


####### convert localized string with date & time to raw datetime
## >> string with date & time in loc.format, for example :'20 Mar 2002 10:23'
## << raw_time
sub str_to_time{
	my $str = shift;

	my $lang_config = $LOCALE{local_lang_conf};
	my $oldlocale = safe_setlocale( &POSIX::LC_ALL, $lang_config->{posixlocale} );

	my $time = Date::Manip::UnixDate($str,"%Y-%m-%d %H:%M");

	reset_locale( &POSIX::LC_ALL, $oldlocale);

	return $time;
}

## returns number of seconds from 1970 begin
## >> raw_time
## << secounds count
sub time_to_secs {
	my $raw_time  = shift;

	return 0 unless $raw_time;

	my $date = &Date::Manip::UnixDate($raw_time, '%m %d %Y %H %M %S');
	return Date::Manip::Date_SecsSince1970( split(' ', $date) );
}

## >> number of seconds from 1970
## << raw_time
sub secs_to_time {
	my $secs  = shift;

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime($secs);
	return (sprintf("%4.4d-%2.2d-%2.2d %2.2d:%2.2d:%2.2d", $year+1900, $mon+1, $mday, $hour, $min, $sec));
}

## this function checks
## >> first datetime
## >> second_datetime
## < result:
##		0 - datetimes are equal,
##		>0 - first datetime is more then second date,
##		<0 - first datetime is less then second date,
sub cmp_times {
	my $first_datetime = shift;
	my $second_datetime = shift;
	my $fist_unixtime = time_to_secs($first_datetime);
	my $second_unixtime = time_to_secs($second_datetime);
	return ($fist_unixtime <=> $second_unixtime);
}
## this function checks
## >> first date
## >> second_date
## < result:
##      0 - datetimes are equal,
##      >0 - first date is more then second date,
##      <0 - first date is less then second date,
sub cmp_dates {
	my $first_datetime = shift;
	my $second_datetime = shift;
	return cmp_times(time_to_date($first_datetime), time_to_date($second_datetime));
}


## >> first_date
## >> second_date
## < diff first_date - second_date in seconds
sub date_diff {
	my $first_datetime = shift;
	my $second_datetime = shift;

	return time_to_secs($first_datetime) - time_to_secs($second_datetime);
}##/date_diff


## Usage:  $min_time = min_time($time1, $time2)
## Description: get ealier time from two args.
## Parameters:
##    time1	>
##    time2	>
## Return value:
##    minimal time.
sub min_time {
	my $first_datetime = shift;
	my $second_datetime = shift;

	my $fist_unixtime = time_to_secs($first_datetime);
	my $second_unixtime = time_to_secs($second_datetime);
	my $result = $fist_unixtime < $second_unixtime ? $first_datetime : $second_datetime;
	return $result;
}

## Usage:  $min_time = max_time($time1, $time2)
## Description: get latest time from two args.
## Parameters:
##    time1	>
##    time2	>
## Return value:
##    minimal time.
sub max_time {
	my $first_datetime = shift;
	my $second_datetime = shift;

	my $fist_unixtime = time_to_secs($first_datetime);
	my $second_unixtime = time_to_secs($second_datetime);
	my $result = $fist_unixtime > $second_unixtime ? $first_datetime : $second_datetime;
	return $result;
}

1;
